/*
 * Access a PWM server to update a display panel backlight.
 *
 * Copyright (C) 2018, 2020, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdlib.h>

#include <ipc/server.h>

#include "backlight_object_server.h"
#include "activation_client.h"
#include "pwm_client.h"



/* Backlight device. */

class BacklightObject_server : public BacklightObject
{
  Activation *_activation;
  PWM *_pwm;
  int _min, _max;

public:
  explicit BacklightObject_server(Activation *activation, PWM *pwm,
                                  int min, int max)
  : _activation(activation), _pwm(pwm), _min(min), _max(max)
  {
  }

  /* Disable the backlight. */

  long disable()
  {
    _activation->disable();
    return L4_EOK;
  }

  /* Enable the backlight. */

  long enable()
  {
    _activation->enable();
    return L4_EOK;
  }

  /* Use the PWM device to update the backlight level. */

  long set_brightness(int level)
  {
    _pwm->set_duty(level);
    return L4_EOK;
  }
};



/* Arguments: <minimum level> <maximum level> */

int main(int argc, char *argv[])
{
  int min, max;

  if (argc < 3)
    return 1;

  /* Obtain a reference to the PWM device. */

  l4_cap_idx_t pwm = l4re_env_get_cap("pwm");

  if (!l4_is_valid_cap(pwm))
    return 1;

  client_Activation activation_obj(pwm);
  client_PWM pwm_obj(pwm);

  /* Initialise and register a new server object. */

  min = atoi(argv[1]);
  max = atoi(argv[2]);

  BacklightObject_server obj(&activation_obj, &pwm_obj, min, max);

  /* Bind and start the IPC server loop. */

  if (ipc_server_loop_for(BacklightObject, &obj, "backlight"))
    return 1;

  return 0;
}
